# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.maps.routing_v2.types import toll_passes as gmr_toll_passes
from google.maps.routing_v2.types import vehicle_info as gmr_vehicle_info

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "RouteModifiers",
    },
)


class RouteModifiers(proto.Message):
    r"""Encapsulates a set of optional conditions to satisfy when
    calculating the routes.

    Attributes:
        avoid_tolls (bool):
            When set to true, avoids toll roads where reasonable, giving
            preference to routes not containing toll roads. Applies only
            to the ``DRIVE`` and ``TWO_WHEELER``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
        avoid_highways (bool):
            When set to true, avoids highways where reasonable, giving
            preference to routes not containing highways. Applies only
            to the ``DRIVE`` and ``TWO_WHEELER``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
        avoid_ferries (bool):
            When set to true, avoids ferries where reasonable, giving
            preference to routes not containing ferries. Applies only to
            the ``DRIVE`` and\ ``TWO_WHEELER``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
        avoid_indoor (bool):
            When set to true, avoids navigating indoors where
            reasonable, giving preference to routes not containing
            indoor navigation. Applies only to the ``WALK``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
        vehicle_info (google.maps.routing_v2.types.VehicleInfo):
            Specifies the vehicle information.
        toll_passes (MutableSequence[google.maps.routing_v2.types.TollPass]):
            Encapsulates information about toll passes. If toll passes
            are provided, the API tries to return the pass price. If
            toll passes are not provided, the API treats the toll pass
            as unknown and tries to return the cash price. Applies only
            to the ``DRIVE`` and ``TWO_WHEELER``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
    """

    avoid_tolls: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    avoid_highways: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    avoid_ferries: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    avoid_indoor: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    vehicle_info: gmr_vehicle_info.VehicleInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        message=gmr_vehicle_info.VehicleInfo,
    )
    toll_passes: MutableSequence[gmr_toll_passes.TollPass] = proto.RepeatedField(
        proto.ENUM,
        number=6,
        enum=gmr_toll_passes.TollPass,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "RoutingPreference",
    },
)


class RoutingPreference(proto.Enum):
    r"""A set of values that specify factors to take into
    consideration when calculating the route.

    Values:
        ROUTING_PREFERENCE_UNSPECIFIED (0):
            No routing preference specified. Default to
            ``TRAFFIC_UNAWARE``.
        TRAFFIC_UNAWARE (1):
            Computes routes without taking live traffic conditions into
            consideration. Suitable when traffic conditions don't matter
            or are not applicable. Using this value produces the lowest
            latency. Note: For
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            ``DRIVE`` and ``TWO_WHEELER``, the route and duration chosen
            are based on road network and average time-independent
            traffic conditions, not current road conditions.
            Consequently, routes may include roads that are temporarily
            closed. Results for a given request may vary over time due
            to changes in the road network, updated average traffic
            conditions, and the distributed nature of the service.
            Results may also vary between nearly-equivalent routes at
            any time or frequency.
        TRAFFIC_AWARE (2):
            Calculates routes taking live traffic conditions into
            consideration. In contrast to ``TRAFFIC_AWARE_OPTIMAL``,
            some optimizations are applied to significantly reduce
            latency.
        TRAFFIC_AWARE_OPTIMAL (3):
            Calculates the routes taking live traffic
            conditions into consideration, without applying
            most performance optimizations. Using this value
            produces the highest latency.
    """
    ROUTING_PREFERENCE_UNSPECIFIED = 0
    TRAFFIC_UNAWARE = 1
    TRAFFIC_AWARE = 2
    TRAFFIC_AWARE_OPTIMAL = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "SpeedReadingInterval",
    },
)


class SpeedReadingInterval(proto.Message):
    r"""Traffic density indicator on a contiguous segment of a polyline or
    path. Given a path with points P_0, P_1, ... , P_N (zero-based
    index), the ``SpeedReadingInterval`` defines an interval and
    describes its traffic using the following categories.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        start_polyline_point_index (int):
            The starting index of this interval in the
            polyline.

            This field is a member of `oneof`_ ``_start_polyline_point_index``.
        end_polyline_point_index (int):
            The ending index of this interval in the
            polyline.

            This field is a member of `oneof`_ ``_end_polyline_point_index``.
        speed (google.maps.routing_v2.types.SpeedReadingInterval.Speed):
            Traffic speed in this interval.

            This field is a member of `oneof`_ ``speed_type``.
    """

    class Speed(proto.Enum):
        r"""The classification of polyline speed based on traffic data.

        Values:
            SPEED_UNSPECIFIED (0):
                Default value. This value is unused.
            NORMAL (1):
                Normal speed, no slowdown is detected.
            SLOW (2):
                Slowdown detected, but no traffic jam formed.
            TRAFFIC_JAM (3):
                Traffic jam detected.
        """
        SPEED_UNSPECIFIED = 0
        NORMAL = 1
        SLOW = 2
        TRAFFIC_JAM = 3

    start_polyline_point_index: int = proto.Field(
        proto.INT32,
        number=1,
        optional=True,
    )
    end_polyline_point_index: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )
    speed: Speed = proto.Field(
        proto.ENUM,
        number=3,
        oneof="speed_type",
        enum=Speed,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "TrafficModel",
    },
)


class TrafficModel(proto.Enum):
    r"""Specifies the assumptions to use when calculating time in traffic.
    This setting affects the value returned in the ``duration`` field in
    the response, which contains the predicted time in traffic based on
    historical averages.

    Values:
        TRAFFIC_MODEL_UNSPECIFIED (0):
            Unused. If specified, will default to ``BEST_GUESS``.
        BEST_GUESS (1):
            Indicates that the returned ``duration`` should be the best
            estimate of travel time given what is known about both
            historical traffic conditions and live traffic. Live traffic
            becomes more important the closer the ``departure_time`` is
            to now.
        PESSIMISTIC (2):
            Indicates that the returned duration should
            be longer than the actual travel time on most
            days, though occasional days with particularly
            bad traffic conditions may exceed this value.
        OPTIMISTIC (3):
            Indicates that the returned duration should
            be shorter than the actual travel time on most
            days, though occasional days with particularly
            good traffic conditions may be faster than this
            value.
    """
    TRAFFIC_MODEL_UNSPECIFIED = 0
    BEST_GUESS = 1
    PESSIMISTIC = 2
    OPTIMISTIC = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "TollPass",
    },
)


class TollPass(proto.Enum):
    r"""List of toll passes around the world that we support.

    Values:
        TOLL_PASS_UNSPECIFIED (0):
            Not used. If this value is used, then the
            request fails.
        AU_ETOLL_TAG (82):
            Sydney toll pass. See additional details at
            https://www.myetoll.com.au.
        AU_EWAY_TAG (83):
            Sydney toll pass. See additional details at
            https://www.tollpay.com.au.
        AU_LINKT (2):
            Australia-wide toll pass.
            See additional details at
            https://www.linkt.com.au/.
        AR_TELEPASE (3):
            Argentina toll pass. See additional details
            at https://telepase.com.ar
        BR_AUTO_EXPRESO (81):
            Brazil toll pass. See additional details at
            https://www.autoexpreso.com
        BR_CONECTCAR (7):
            Brazil toll pass. See additional details at
            https://conectcar.com.
        BR_MOVE_MAIS (8):
            Brazil toll pass. See additional details at
            https://movemais.com.
        BR_PASSA_RAPIDO (88):
            Brazil toll pass. See additional details at
            https://pasorapido.gob.do/
        BR_SEM_PARAR (9):
            Brazil toll pass. See additional details at
            https://www.semparar.com.br.
        BR_TAGGY (10):
            Brazil toll pass. See additional details at
            https://taggy.com.br.
        BR_VELOE (11):
            Brazil toll pass. See additional details at
            https://veloe.com.br/site/onde-usar.
        CA_US_AKWASASNE_SEAWAY_CORPORATE_CARD (84):
            Canada to United States border crossing.
        CA_US_AKWASASNE_SEAWAY_TRANSIT_CARD (85):
            Canada to United States border crossing.
        CA_US_BLUE_WATER_EDGE_PASS (18):
            Ontario, Canada to Michigan, United States
            border crossing.
        CA_US_CONNEXION (19):
            Ontario, Canada to Michigan, United States
            border crossing.
        CA_US_NEXUS_CARD (20):
            Canada to United States border crossing.
        ID_E_TOLL (16):
            Indonesia.
            E-card provided by multiple banks used to pay
            for tolls. All e-cards via banks are charged the
            same so only one enum value is needed. E.g.

            - Bank Mandiri
              https://www.bankmandiri.co.id/e-money
            - BCA https://www.bca.co.id/flazz
            - BNI
              https://www.bni.co.id/id-id/ebanking/tapcash
        IN_FASTAG (78):
            India.
        IN_LOCAL_HP_PLATE_EXEMPT (79):
            India, HP state plate exemption.
        JP_ETC (98):
            Japan
            ETC. Electronic wireless system to collect
            tolls. https://www.go-etc.jp/
        JP_ETC2 (99):
            Japan
            ETC2.0. New version of ETC with further discount
            and bidirectional communication between devices
            on vehicles and antennas on the road.
            https://www.go-etc.jp/etc2/index.html
        MX_IAVE (90):
            Mexico toll pass.
            https://iave.capufe.gob.mx/#/
        MX_PASE (91):
            Mexico
            https://www.pase.com.mx
        MX_QUICKPASS (93):
            Mexico
            https://operadoravial.com/quick-pass/
        MX_SISTEMA_TELEPEAJE_CHIHUAHUA (89):
            http://appsh.chihuahua.gob.mx/transparencia/?doc=/ingresos/TelepeajeFormato4.pdf
        MX_TAG_IAVE (12):
            Mexico
        MX_TAG_TELEVIA (13):
            Mexico toll pass company. One of many
            operating in Mexico City. See additional details
            at https://www.televia.com.mx.
        MX_TELEVIA (92):
            Mexico toll pass company. One of many
            operating in Mexico City.
            https://www.televia.com.mx
        MX_VIAPASS (14):
            Mexico toll pass. See additional details at
            https://www.viapass.com.mx/viapass/web_home.aspx.
        US_AL_FREEDOM_PASS (21):
            AL, USA.
        US_AK_ANTON_ANDERSON_TUNNEL_BOOK_OF_10_TICKETS (22):
            AK, USA.
        US_CA_FASTRAK (4):
            CA, USA.
        US_CA_FASTRAK_CAV_STICKER (86):
            Indicates driver has any FasTrak pass in
            addition to the DMV issued Clean Air Vehicle
            (CAV) sticker.
            https://www.bayareafastrak.org/en/guide/doINeedFlex.shtml
        US_CO_EXPRESSTOLL (23):
            CO, USA.
        US_CO_GO_PASS (24):
            CO, USA.
        US_DE_EZPASSDE (25):
            DE, USA.
        US_FL_BOB_SIKES_TOLL_BRIDGE_PASS (65):
            FL, USA.
        US_FL_DUNES_COMMUNITY_DEVELOPMENT_DISTRICT_EXPRESSCARD (66):
            FL, USA.
        US_FL_EPASS (67):
            FL, USA.
        US_FL_GIBA_TOLL_PASS (68):
            FL, USA.
        US_FL_LEEWAY (69):
            FL, USA.
        US_FL_SUNPASS (70):
            FL, USA.
        US_FL_SUNPASS_PRO (71):
            FL, USA.
        US_IL_EZPASSIL (73):
            IL, USA.
        US_IL_IPASS (72):
            IL, USA.
        US_IN_EZPASSIN (26):
            IN, USA.
        US_KS_BESTPASS_HORIZON (27):
            KS, USA.
        US_KS_KTAG (28):
            KS, USA.
        US_KS_NATIONALPASS (29):
            KS, USA.
        US_KS_PREPASS_ELITEPASS (30):
            KS, USA.
        US_KY_RIVERLINK (31):
            KY, USA.
        US_LA_GEAUXPASS (32):
            LA, USA.
        US_LA_TOLL_TAG (33):
            LA, USA.
        US_MA_EZPASSMA (6):
            MA, USA.
        US_MD_EZPASSMD (34):
            MD, USA.
        US_ME_EZPASSME (35):
            ME, USA.
        US_MI_AMBASSADOR_BRIDGE_PREMIER_COMMUTER_CARD (36):
            MI, USA.
        US_MI_BCPASS (94):
            MI, USA.
        US_MI_GROSSE_ILE_TOLL_BRIDGE_PASS_TAG (37):
            MI, USA.
        US_MI_IQ_PROX_CARD (38):
            MI, USA.
            Deprecated as this pass type no longer exists.
        US_MI_IQ_TAG (95):
            MI, USA.
        US_MI_MACKINAC_BRIDGE_MAC_PASS (39):
            MI, USA.
        US_MI_NEXPRESS_TOLL (40):
            MI, USA.
        US_MN_EZPASSMN (41):
            MN, USA.
        US_NC_EZPASSNC (42):
            NC, USA.
        US_NC_PEACH_PASS (87):
            NC, USA.
        US_NC_QUICK_PASS (43):
            NC, USA.
        US_NH_EZPASSNH (80):
            NH, USA.
        US_NJ_DOWNBEACH_EXPRESS_PASS (75):
            NJ, USA.
        US_NJ_EZPASSNJ (74):
            NJ, USA.
        US_NY_EXPRESSPASS (76):
            NY, USA.
        US_NY_EZPASSNY (77):
            NY, USA.
        US_OH_EZPASSOH (44):
            OH, USA.
        US_PA_EZPASSPA (45):
            PA, USA.
        US_RI_EZPASSRI (46):
            RI, USA.
        US_SC_PALPASS (47):
            SC, USA.
        US_TX_AVI_TAG (97):
            TX, USA.
        US_TX_BANCPASS (48):
            TX, USA.
        US_TX_DEL_RIO_PASS (49):
            TX, USA.
        US_TX_EFAST_PASS (50):
            TX, USA.
        US_TX_EAGLE_PASS_EXPRESS_CARD (51):
            TX, USA.
        US_TX_EPTOLL (52):
            TX, USA.
        US_TX_EZ_CROSS (53):
            TX, USA.
        US_TX_EZTAG (54):
            TX, USA.
        US_TX_FUEGO_TAG (96):
            TX, USA.
        US_TX_LAREDO_TRADE_TAG (55):
            TX, USA.
        US_TX_PLUSPASS (56):
            TX, USA.
        US_TX_TOLLTAG (57):
            TX, USA.
        US_TX_TXTAG (58):
            TX, USA.
        US_TX_XPRESS_CARD (59):
            TX, USA.
        US_UT_ADAMS_AVE_PARKWAY_EXPRESSCARD (60):
            UT, USA.
        US_VA_EZPASSVA (61):
            VA, USA.
        US_WA_BREEZEBY (17):
            WA, USA.
        US_WA_GOOD_TO_GO (1):
            WA, USA.
        US_WV_EZPASSWV (62):
            WV, USA.
        US_WV_MEMORIAL_BRIDGE_TICKETS (63):
            WV, USA.
        US_WV_MOV_PASS (100):
            WV, USA
        US_WV_NEWELL_TOLL_BRIDGE_TICKET (64):
            WV, USA.
    """
    TOLL_PASS_UNSPECIFIED = 0
    AU_ETOLL_TAG = 82
    AU_EWAY_TAG = 83
    AU_LINKT = 2
    AR_TELEPASE = 3
    BR_AUTO_EXPRESO = 81
    BR_CONECTCAR = 7
    BR_MOVE_MAIS = 8
    BR_PASSA_RAPIDO = 88
    BR_SEM_PARAR = 9
    BR_TAGGY = 10
    BR_VELOE = 11
    CA_US_AKWASASNE_SEAWAY_CORPORATE_CARD = 84
    CA_US_AKWASASNE_SEAWAY_TRANSIT_CARD = 85
    CA_US_BLUE_WATER_EDGE_PASS = 18
    CA_US_CONNEXION = 19
    CA_US_NEXUS_CARD = 20
    ID_E_TOLL = 16
    IN_FASTAG = 78
    IN_LOCAL_HP_PLATE_EXEMPT = 79
    JP_ETC = 98
    JP_ETC2 = 99
    MX_IAVE = 90
    MX_PASE = 91
    MX_QUICKPASS = 93
    MX_SISTEMA_TELEPEAJE_CHIHUAHUA = 89
    MX_TAG_IAVE = 12
    MX_TAG_TELEVIA = 13
    MX_TELEVIA = 92
    MX_VIAPASS = 14
    US_AL_FREEDOM_PASS = 21
    US_AK_ANTON_ANDERSON_TUNNEL_BOOK_OF_10_TICKETS = 22
    US_CA_FASTRAK = 4
    US_CA_FASTRAK_CAV_STICKER = 86
    US_CO_EXPRESSTOLL = 23
    US_CO_GO_PASS = 24
    US_DE_EZPASSDE = 25
    US_FL_BOB_SIKES_TOLL_BRIDGE_PASS = 65
    US_FL_DUNES_COMMUNITY_DEVELOPMENT_DISTRICT_EXPRESSCARD = 66
    US_FL_EPASS = 67
    US_FL_GIBA_TOLL_PASS = 68
    US_FL_LEEWAY = 69
    US_FL_SUNPASS = 70
    US_FL_SUNPASS_PRO = 71
    US_IL_EZPASSIL = 73
    US_IL_IPASS = 72
    US_IN_EZPASSIN = 26
    US_KS_BESTPASS_HORIZON = 27
    US_KS_KTAG = 28
    US_KS_NATIONALPASS = 29
    US_KS_PREPASS_ELITEPASS = 30
    US_KY_RIVERLINK = 31
    US_LA_GEAUXPASS = 32
    US_LA_TOLL_TAG = 33
    US_MA_EZPASSMA = 6
    US_MD_EZPASSMD = 34
    US_ME_EZPASSME = 35
    US_MI_AMBASSADOR_BRIDGE_PREMIER_COMMUTER_CARD = 36
    US_MI_BCPASS = 94
    US_MI_GROSSE_ILE_TOLL_BRIDGE_PASS_TAG = 37
    US_MI_IQ_PROX_CARD = 38
    US_MI_IQ_TAG = 95
    US_MI_MACKINAC_BRIDGE_MAC_PASS = 39
    US_MI_NEXPRESS_TOLL = 40
    US_MN_EZPASSMN = 41
    US_NC_EZPASSNC = 42
    US_NC_PEACH_PASS = 87
    US_NC_QUICK_PASS = 43
    US_NH_EZPASSNH = 80
    US_NJ_DOWNBEACH_EXPRESS_PASS = 75
    US_NJ_EZPASSNJ = 74
    US_NY_EXPRESSPASS = 76
    US_NY_EZPASSNY = 77
    US_OH_EZPASSOH = 44
    US_PA_EZPASSPA = 45
    US_RI_EZPASSRI = 46
    US_SC_PALPASS = 47
    US_TX_AVI_TAG = 97
    US_TX_BANCPASS = 48
    US_TX_DEL_RIO_PASS = 49
    US_TX_EFAST_PASS = 50
    US_TX_EAGLE_PASS_EXPRESS_CARD = 51
    US_TX_EPTOLL = 52
    US_TX_EZ_CROSS = 53
    US_TX_EZTAG = 54
    US_TX_FUEGO_TAG = 96
    US_TX_LAREDO_TRADE_TAG = 55
    US_TX_PLUSPASS = 56
    US_TX_TOLLTAG = 57
    US_TX_TXTAG = 58
    US_TX_XPRESS_CARD = 59
    US_UT_ADAMS_AVE_PARKWAY_EXPRESSCARD = 60
    US_VA_EZPASSVA = 61
    US_WA_BREEZEBY = 17
    US_WA_GOOD_TO_GO = 1
    US_WV_EZPASSWV = 62
    US_WV_MEMORIAL_BRIDGE_TICKETS = 63
    US_WV_MOV_PASS = 100
    US_WV_NEWELL_TOLL_BRIDGE_TICKET = 64


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "VehicleEmissionType",
    },
)


class VehicleEmissionType(proto.Enum):
    r"""A set of values describing the vehicle's emission type. Applies only
    to the ``DRIVE``
    [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].

    Values:
        VEHICLE_EMISSION_TYPE_UNSPECIFIED (0):
            No emission type specified. Default to ``GASOLINE``.
        GASOLINE (1):
            Gasoline/petrol fueled vehicle.
        ELECTRIC (2):
            Electricity powered vehicle.
        HYBRID (3):
            Hybrid fuel (such as gasoline + electric)
            vehicle.
        DIESEL (4):
            Diesel fueled vehicle.
    """
    VEHICLE_EMISSION_TYPE_UNSPECIFIED = 0
    GASOLINE = 1
    ELECTRIC = 2
    HYBRID = 3
    DIESEL = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.type import localized_text_pb2  # type: ignore
import proto  # type: ignore

from google.maps.routing_v2.types import location as gmr_location

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "TransitAgency",
        "TransitLine",
        "TransitStop",
        "TransitVehicle",
    },
)


class TransitAgency(proto.Message):
    r"""A transit agency that operates a transit line.

    Attributes:
        name (str):
            The name of this transit agency.
        phone_number (str):
            The transit agency's locale-specific
            formatted phone number.
        uri (str):
            The transit agency's URI.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    phone_number: str = proto.Field(
        proto.STRING,
        number=2,
    )
    uri: str = proto.Field(
        proto.STRING,
        number=3,
    )


class TransitLine(proto.Message):
    r"""Contains information about the transit line used in this
    step.

    Attributes:
        agencies (MutableSequence[google.maps.routing_v2.types.TransitAgency]):
            The transit agency (or agencies) that
            operates this transit line.
        name (str):
            The full name of this transit line, For
            example, "8 Avenue Local".
        uri (str):
            the URI for this transit line as provided by
            the transit agency.
        color (str):
            The color commonly used in signage for this
            line. Represented in hexadecimal.
        icon_uri (str):
            The URI for the icon associated with this
            line.
        name_short (str):
            The short name of this transit line. This
            name will normally be a line number, such as
            "M7" or "355".
        text_color (str):
            The color commonly used in text on signage
            for this line. Represented in hexadecimal.
        vehicle (google.maps.routing_v2.types.TransitVehicle):
            The type of vehicle that operates on this
            transit line.
    """

    agencies: MutableSequence["TransitAgency"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="TransitAgency",
    )
    name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    uri: str = proto.Field(
        proto.STRING,
        number=3,
    )
    color: str = proto.Field(
        proto.STRING,
        number=4,
    )
    icon_uri: str = proto.Field(
        proto.STRING,
        number=5,
    )
    name_short: str = proto.Field(
        proto.STRING,
        number=6,
    )
    text_color: str = proto.Field(
        proto.STRING,
        number=7,
    )
    vehicle: "TransitVehicle" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="TransitVehicle",
    )


class TransitStop(proto.Message):
    r"""Information about a transit stop.

    Attributes:
        name (str):
            The name of the transit stop.
        location (google.maps.routing_v2.types.Location):
            The location of the stop expressed in
            latitude/longitude coordinates.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    location: gmr_location.Location = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gmr_location.Location,
    )


class TransitVehicle(proto.Message):
    r"""Information about a vehicle used in transit routes.

    Attributes:
        name (google.type.localized_text_pb2.LocalizedText):
            The name of this vehicle, capitalized.
        type_ (google.maps.routing_v2.types.TransitVehicle.TransitVehicleType):
            The type of vehicle used.
        icon_uri (str):
            The URI for an icon associated with this
            vehicle type.
        local_icon_uri (str):
            The URI for the icon associated with this
            vehicle type, based on the local transport
            signage.
    """

    class TransitVehicleType(proto.Enum):
        r"""The type of vehicles for transit routes.

        Values:
            TRANSIT_VEHICLE_TYPE_UNSPECIFIED (0):
                Unused.
            BUS (1):
                Bus.
            CABLE_CAR (2):
                A vehicle that operates on a cable, usually on the ground.
                Aerial cable cars may be of the type ``GONDOLA_LIFT``.
            COMMUTER_TRAIN (3):
                Commuter rail.
            FERRY (4):
                Ferry.
            FUNICULAR (5):
                A vehicle that is pulled up a steep incline
                by a cable. A Funicular typically consists of
                two cars, with each car acting as a
                counterweight for the other.
            GONDOLA_LIFT (6):
                An aerial cable car.
            HEAVY_RAIL (7):
                Heavy rail.
            HIGH_SPEED_TRAIN (8):
                High speed train.
            INTERCITY_BUS (9):
                Intercity bus.
            LONG_DISTANCE_TRAIN (10):
                Long distance train.
            METRO_RAIL (11):
                Light rail transit.
            MONORAIL (12):
                Monorail.
            OTHER (13):
                All other vehicles.
            RAIL (14):
                Rail.
            SHARE_TAXI (15):
                Share taxi is a kind of bus with the ability
                to drop off and pick up passengers anywhere on
                its route.
            SUBWAY (16):
                Underground light rail.
            TRAM (17):
                Above ground light rail.
            TROLLEYBUS (18):
                Trolleybus.
        """
        TRANSIT_VEHICLE_TYPE_UNSPECIFIED = 0
        BUS = 1
        CABLE_CAR = 2
        COMMUTER_TRAIN = 3
        FERRY = 4
        FUNICULAR = 5
        GONDOLA_LIFT = 6
        HEAVY_RAIL = 7
        HIGH_SPEED_TRAIN = 8
        INTERCITY_BUS = 9
        LONG_DISTANCE_TRAIN = 10
        METRO_RAIL = 11
        MONORAIL = 12
        OTHER = 13
        RAIL = 14
        SHARE_TAXI = 15
        SUBWAY = 16
        TRAM = 17
        TROLLEYBUS = 18

    name: localized_text_pb2.LocalizedText = proto.Field(
        proto.MESSAGE,
        number=1,
        message=localized_text_pb2.LocalizedText,
    )
    type_: TransitVehicleType = proto.Field(
        proto.ENUM,
        number=2,
        enum=TransitVehicleType,
    )
    icon_uri: str = proto.Field(
        proto.STRING,
        number=3,
    )
    local_icon_uri: str = proto.Field(
        proto.STRING,
        number=4,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore
from google.type import localized_text_pb2  # type: ignore
import proto  # type: ignore

from google.maps.routing_v2.types import geocoding_results as gmr_geocoding_results
from google.maps.routing_v2.types import routing_preference as gmr_routing_preference
from google.maps.routing_v2.types import transit_preferences as gmr_transit_preferences
from google.maps.routing_v2.types import fallback_info as gmr_fallback_info
from google.maps.routing_v2.types import polyline, route
from google.maps.routing_v2.types import route_modifiers as gmr_route_modifiers
from google.maps.routing_v2.types import route_travel_mode
from google.maps.routing_v2.types import traffic_model as gmr_traffic_model
from google.maps.routing_v2.types import units as gmr_units
from google.maps.routing_v2.types import waypoint as gmr_waypoint

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "RouteMatrixElementCondition",
        "ComputeRoutesRequest",
        "ComputeRoutesResponse",
        "ComputeRouteMatrixRequest",
        "RouteMatrixOrigin",
        "RouteMatrixDestination",
        "RouteMatrixElement",
    },
)


class RouteMatrixElementCondition(proto.Enum):
    r"""The condition of the route being returned.

    Values:
        ROUTE_MATRIX_ELEMENT_CONDITION_UNSPECIFIED (0):
            Only used when the ``status`` of the element is not OK.
        ROUTE_EXISTS (1):
            A route was found, and the corresponding
            information was filled out for the element.
        ROUTE_NOT_FOUND (2):
            No route could be found. Fields containing route
            information, such as ``distance_meters`` or ``duration``,
            will not be filled out in the element.
    """
    ROUTE_MATRIX_ELEMENT_CONDITION_UNSPECIFIED = 0
    ROUTE_EXISTS = 1
    ROUTE_NOT_FOUND = 2


class ComputeRoutesRequest(proto.Message):
    r"""ComputeRoutes request message.

    Attributes:
        origin (google.maps.routing_v2.types.Waypoint):
            Required. Origin waypoint.
        destination (google.maps.routing_v2.types.Waypoint):
            Required. Destination waypoint.
        intermediates (MutableSequence[google.maps.routing_v2.types.Waypoint]):
            Optional. A set of waypoints along the route
            (excluding terminal points), for either stopping
            at or passing by. Up to 25 intermediate
            waypoints are supported.
        travel_mode (google.maps.routing_v2.types.RouteTravelMode):
            Optional. Specifies the mode of
            transportation.
        routing_preference (google.maps.routing_v2.types.RoutingPreference):
            Optional. Specifies how to compute the route. The server
            attempts to use the selected routing preference to compute
            the route. If the routing preference results in an error or
            an extra long latency, then an error is returned. You can
            specify this option only when the ``travel_mode`` is
            ``DRIVE`` or ``TWO_WHEELER``, otherwise the request fails.
        polyline_quality (google.maps.routing_v2.types.PolylineQuality):
            Optional. Specifies your preference for the
            quality of the polyline.
        polyline_encoding (google.maps.routing_v2.types.PolylineEncoding):
            Optional. Specifies the preferred encoding
            for the polyline.
        departure_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The departure time. If you don't set this value,
            then this value defaults to the time that you made the
            request. NOTE: You can only specify a ``departure_time`` in
            the past when
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            is set to ``TRANSIT``. Transit trips are available for up to
            7 days in the past or 100 days in the future.
        arrival_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The arrival time. NOTE: Can only be set when
            [RouteTravelMode][google.maps.routing.v2.RouteTravelMode] is
            set to ``TRANSIT``. You can specify either
            ``departure_time`` or ``arrival_time``, but not both.
            Transit trips are available for up to 7 days in the past or
            100 days in the future.
        compute_alternative_routes (bool):
            Optional. Specifies whether to calculate
            alternate routes in addition to the route. No
            alternative routes are returned for requests
            that have intermediate waypoints.
        route_modifiers (google.maps.routing_v2.types.RouteModifiers):
            Optional. A set of conditions to satisfy that
            affect the way routes are calculated.
        language_code (str):
            Optional. The BCP-47 language code, such as "en-US" or
            "sr-Latn". For more information, see `Unicode Locale
            Identifier <http://www.unicode.org/reports/tr35/#Unicode_locale_identifier>`__.
            See `Language
            Support <https://developers.google.com/maps/faq#languagesupport>`__
            for the list of supported languages. When you don't provide
            this value, the display language is inferred from the
            location of the route request.
        region_code (str):
            Optional. The region code, specified as a ccTLD ("top-level
            domain") two-character value. For more information see
            `Country code top-level
            domains <https://en.wikipedia.org/wiki/List_of_Internet_top-level_domains#Country_code_top-level_domains>`__.
        units (google.maps.routing_v2.types.Units):
            Optional. Specifies the units of measure for the display
            fields. These fields include the ``instruction`` field in
            [``NavigationInstruction``][google.maps.routing.v2.NavigationInstruction].
            The units of measure used for the route, leg, step distance,
            and duration are not affected by this value. If you don't
            provide this value, then the display units are inferred from
            the location of the first origin.
        optimize_waypoint_order (bool):
            Optional. If set to true, the service attempts to minimize
            the overall cost of the route by re-ordering the specified
            intermediate waypoints. The request fails if any of the
            intermediate waypoints is a ``via`` waypoint. Use
            ``ComputeRoutesResponse.Routes.optimized_intermediate_waypoint_index``
            to find the new ordering. If
            ``ComputeRoutesResponseroutes.optimized_intermediate_waypoint_index``
            is not requested in the ``X-Goog-FieldMask`` header, the
            request fails. If ``optimize_waypoint_order`` is set to
            false,
            ``ComputeRoutesResponse.optimized_intermediate_waypoint_index``
            will be empty.
        requested_reference_routes (MutableSequence[google.maps.routing_v2.types.ComputeRoutesRequest.ReferenceRoute]):
            Optional. Specifies what reference routes to calculate as
            part of the request in addition to the default route. A
            reference route is a route with a different route
            calculation objective than the default route. For example a
            ``FUEL_EFFICIENT`` reference route calculation takes into
            account various parameters that would generate an optimal
            fuel efficient route.
        extra_computations (MutableSequence[google.maps.routing_v2.types.ComputeRoutesRequest.ExtraComputation]):
            Optional. A list of extra computations which
            may be used to complete the request. Note: These
            extra computations may return extra fields on
            the response. These extra fields must also be
            specified in the field mask to be returned in
            the response.
        traffic_model (google.maps.routing_v2.types.TrafficModel):
            Optional. Specifies the assumptions to use when calculating
            time in traffic. This setting affects the value returned in
            the duration field in the
            [``Route``][google.maps.routing.v2.Route] and
            [``RouteLeg``][google.maps.routing.v2.RouteLeg] which
            contains the predicted time in traffic based on historical
            averages. ``TrafficModel`` is only available for requests
            that have set
            [``RoutingPreference``][google.maps.routing.v2.RoutingPreference]
            to ``TRAFFIC_AWARE_OPTIMAL`` and
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            to ``DRIVE``. Defaults to ``BEST_GUESS`` if traffic is
            requested and ``TrafficModel`` is not specified.
        transit_preferences (google.maps.routing_v2.types.TransitPreferences):
            Optional. Specifies preferences that influence the route
            returned for ``TRANSIT`` routes. NOTE: You can only specify
            a ``transit_preferences`` when
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            is set to ``TRANSIT``.
    """

    class ReferenceRoute(proto.Enum):
        r"""A supported reference route on the ComputeRoutesRequest.

        Values:
            REFERENCE_ROUTE_UNSPECIFIED (0):
                Not used. Requests containing this value
                fail.
            FUEL_EFFICIENT (1):
                Fuel efficient route. Routes labeled with
                this value are determined to be optimized for
                parameters such as fuel consumption.
        """
        REFERENCE_ROUTE_UNSPECIFIED = 0
        FUEL_EFFICIENT = 1

    class ExtraComputation(proto.Enum):
        r"""Extra computations to perform while completing the request.

        Values:
            EXTRA_COMPUTATION_UNSPECIFIED (0):
                Not used. Requests containing this value will
                fail.
            TOLLS (1):
                Toll information for the route(s).
            FUEL_CONSUMPTION (2):
                Estimated fuel consumption for the route(s).
            TRAFFIC_ON_POLYLINE (3):
                Traffic aware polylines for the route(s).
            HTML_FORMATTED_NAVIGATION_INSTRUCTIONS (4):
                ```NavigationInstructions`` <google.maps.routing.v2.NavigationInstructions.instructions>`__
                presented as a formatted HTML text string. This content is
                meant to be read as-is. This content is for display only. Do
                not programmatically parse it.
        """
        EXTRA_COMPUTATION_UNSPECIFIED = 0
        TOLLS = 1
        FUEL_CONSUMPTION = 2
        TRAFFIC_ON_POLYLINE = 3
        HTML_FORMATTED_NAVIGATION_INSTRUCTIONS = 4

    origin: gmr_waypoint.Waypoint = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gmr_waypoint.Waypoint,
    )
    destination: gmr_waypoint.Waypoint = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gmr_waypoint.Waypoint,
    )
    intermediates: MutableSequence[gmr_waypoint.Waypoint] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=gmr_waypoint.Waypoint,
    )
    travel_mode: route_travel_mode.RouteTravelMode = proto.Field(
        proto.ENUM,
        number=4,
        enum=route_travel_mode.RouteTravelMode,
    )
    routing_preference: gmr_routing_preference.RoutingPreference = proto.Field(
        proto.ENUM,
        number=5,
        enum=gmr_routing_preference.RoutingPreference,
    )
    polyline_quality: polyline.PolylineQuality = proto.Field(
        proto.ENUM,
        number=6,
        enum=polyline.PolylineQuality,
    )
    polyline_encoding: polyline.PolylineEncoding = proto.Field(
        proto.ENUM,
        number=12,
        enum=polyline.PolylineEncoding,
    )
    departure_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=7,
        message=timestamp_pb2.Timestamp,
    )
    arrival_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=19,
        message=timestamp_pb2.Timestamp,
    )
    compute_alternative_routes: bool = proto.Field(
        proto.BOOL,
        number=8,
    )
    route_modifiers: gmr_route_modifiers.RouteModifiers = proto.Field(
        proto.MESSAGE,
        number=9,
        message=gmr_route_modifiers.RouteModifiers,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=10,
    )
    region_code: str = proto.Field(
        proto.STRING,
        number=16,
    )
    units: gmr_units.Units = proto.Field(
        proto.ENUM,
        number=11,
        enum=gmr_units.Units,
    )
    optimize_waypoint_order: bool = proto.Field(
        proto.BOOL,
        number=13,
    )
    requested_reference_routes: MutableSequence[ReferenceRoute] = proto.RepeatedField(
        proto.ENUM,
        number=14,
        enum=ReferenceRoute,
    )
    extra_computations: MutableSequence[ExtraComputation] = proto.RepeatedField(
        proto.ENUM,
        number=15,
        enum=ExtraComputation,
    )
    traffic_model: gmr_traffic_model.TrafficModel = proto.Field(
        proto.ENUM,
        number=18,
        enum=gmr_traffic_model.TrafficModel,
    )
    transit_preferences: gmr_transit_preferences.TransitPreferences = proto.Field(
        proto.MESSAGE,
        number=20,
        message=gmr_transit_preferences.TransitPreferences,
    )


class ComputeRoutesResponse(proto.Message):
    r"""ComputeRoutes the response message.

    Attributes:
        routes (MutableSequence[google.maps.routing_v2.types.Route]):
            Contains an array of computed routes (up to three) when you
            specify ``compute_alternatives_routes``, and contains just
            one route when you don't. When this array contains multiple
            entries, the first one is the most recommended route. If the
            array is empty, then it means no route could be found.
        fallback_info (google.maps.routing_v2.types.FallbackInfo):
            In some cases when the server is not able to
            compute the route results with all of the input
            preferences, it may fallback to using a
            different way of computation. When fallback mode
            is used, this field contains detailed info about
            the fallback response. Otherwise this field is
            unset.
        geocoding_results (google.maps.routing_v2.types.GeocodingResults):
            Contains geocoding response info for
            waypoints specified as addresses.
    """

    routes: MutableSequence[route.Route] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=route.Route,
    )
    fallback_info: gmr_fallback_info.FallbackInfo = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gmr_fallback_info.FallbackInfo,
    )
    geocoding_results: gmr_geocoding_results.GeocodingResults = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gmr_geocoding_results.GeocodingResults,
    )


class ComputeRouteMatrixRequest(proto.Message):
    r"""ComputeRouteMatrix request message

    Attributes:
        origins (MutableSequence[google.maps.routing_v2.types.RouteMatrixOrigin]):
            Required. Array of origins, which determines the rows of the
            response matrix. Several size restrictions apply to the
            cardinality of origins and destinations:

            -  The sum of the number of origins + the number of
               destinations specified as either ``place_id`` or
               ``address`` must be no greater than 50.
            -  The product of number of origins × number of destinations
               must be no greater than 625 in any case.
            -  The product of the number of origins × number of
               destinations must be no greater than 100 if
               routing_preference is set to ``TRAFFIC_AWARE_OPTIMAL``.
            -  The product of the number of origins × number of
               destinations must be no greater than 100 if travel_mode
               is set to ``TRANSIT``.
        destinations (MutableSequence[google.maps.routing_v2.types.RouteMatrixDestination]):
            Required. Array of destinations, which
            determines the columns of the response matrix.
        travel_mode (google.maps.routing_v2.types.RouteTravelMode):
            Optional. Specifies the mode of
            transportation.
        routing_preference (google.maps.routing_v2.types.RoutingPreference):
            Optional. Specifies how to compute the route. The server
            attempts to use the selected routing preference to compute
            the route. If the routing preference results in an error or
            an extra long latency, an error is returned. You can specify
            this option only when the ``travel_mode`` is ``DRIVE`` or
            ``TWO_WHEELER``, otherwise the request fails.
        departure_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The departure time. If you don't set this value,
            then this value defaults to the time that you made the
            request. NOTE: You can only specify a ``departure_time`` in
            the past when
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            is set to ``TRANSIT``.
        arrival_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The arrival time. NOTE: Can only be set when
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode]
            is set to ``TRANSIT``. You can specify either
            ``departure_time`` or ``arrival_time``, but not both.
        language_code (str):
            Optional. The BCP-47 language code, such as "en-US" or
            "sr-Latn". For more information, see `Unicode Locale
            Identifier <http://www.unicode.org/reports/tr35/#Unicode_locale_identifier>`__.
            See `Language
            Support <https://developers.google.com/maps/faq#languagesupport>`__
            for the list of supported languages. When you don't provide
            this value, the display language is inferred from the
            location of the first origin.
        region_code (str):
            Optional. The region code, specified as a ccTLD ("top-level
            domain") two-character value. For more information see
            `Country code top-level
            domains <https://en.wikipedia.org/wiki/List_of_Internet_top-level_domains#Country_code_top-level_domains>`__.
        units (google.maps.routing_v2.types.Units):
            Optional. Specifies the units of measure for
            the display fields.
        extra_computations (MutableSequence[google.maps.routing_v2.types.ComputeRouteMatrixRequest.ExtraComputation]):
            Optional. A list of extra computations which
            may be used to complete the request. Note: These
            extra computations may return extra fields on
            the response. These extra fields must also be
            specified in the field mask to be returned in
            the response.
        traffic_model (google.maps.routing_v2.types.TrafficModel):
            Optional. Specifies the assumptions to use when calculating
            time in traffic. This setting affects the value returned in
            the duration field in the
            [RouteMatrixElement][google.maps.routing.v2.RouteMatrixElement]
            which contains the predicted time in traffic based on
            historical averages.
            [RoutingPreference][google.maps.routing.v2.RoutingPreference]
            to ``TRAFFIC_AWARE_OPTIMAL`` and
            [RouteTravelMode][google.maps.routing.v2.RouteTravelMode] to
            ``DRIVE``. Defaults to ``BEST_GUESS`` if traffic is
            requested and ``TrafficModel`` is not specified.
        transit_preferences (google.maps.routing_v2.types.TransitPreferences):
            Optional. Specifies preferences that influence the route
            returned for ``TRANSIT`` routes. NOTE: You can only specify
            a ``transit_preferences`` when
            [RouteTravelMode][google.maps.routing.v2.RouteTravelMode] is
            set to ``TRANSIT``.
    """

    class ExtraComputation(proto.Enum):
        r"""Extra computations to perform while completing the request.

        Values:
            EXTRA_COMPUTATION_UNSPECIFIED (0):
                Not used. Requests containing this value will
                fail.
            TOLLS (1):
                Toll information for the matrix element(s).
        """
        EXTRA_COMPUTATION_UNSPECIFIED = 0
        TOLLS = 1

    origins: MutableSequence["RouteMatrixOrigin"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="RouteMatrixOrigin",
    )
    destinations: MutableSequence["RouteMatrixDestination"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="RouteMatrixDestination",
    )
    travel_mode: route_travel_mode.RouteTravelMode = proto.Field(
        proto.ENUM,
        number=3,
        enum=route_travel_mode.RouteTravelMode,
    )
    routing_preference: gmr_routing_preference.RoutingPreference = proto.Field(
        proto.ENUM,
        number=4,
        enum=gmr_routing_preference.RoutingPreference,
    )
    departure_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    arrival_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=6,
    )
    region_code: str = proto.Field(
        proto.STRING,
        number=9,
    )
    units: gmr_units.Units = proto.Field(
        proto.ENUM,
        number=7,
        enum=gmr_units.Units,
    )
    extra_computations: MutableSequence[ExtraComputation] = proto.RepeatedField(
        proto.ENUM,
        number=8,
        enum=ExtraComputation,
    )
    traffic_model: gmr_traffic_model.TrafficModel = proto.Field(
        proto.ENUM,
        number=10,
        enum=gmr_traffic_model.TrafficModel,
    )
    transit_preferences: gmr_transit_preferences.TransitPreferences = proto.Field(
        proto.MESSAGE,
        number=12,
        message=gmr_transit_preferences.TransitPreferences,
    )


class RouteMatrixOrigin(proto.Message):
    r"""A single origin for ComputeRouteMatrixRequest

    Attributes:
        waypoint (google.maps.routing_v2.types.Waypoint):
            Required. Origin waypoint
        route_modifiers (google.maps.routing_v2.types.RouteModifiers):
            Optional. Modifiers for every route that
            takes this as the origin
    """

    waypoint: gmr_waypoint.Waypoint = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gmr_waypoint.Waypoint,
    )
    route_modifiers: gmr_route_modifiers.RouteModifiers = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gmr_route_modifiers.RouteModifiers,
    )


class RouteMatrixDestination(proto.Message):
    r"""A single destination for ComputeRouteMatrixRequest

    Attributes:
        waypoint (google.maps.routing_v2.types.Waypoint):
            Required. Destination waypoint
    """

    waypoint: gmr_waypoint.Waypoint = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gmr_waypoint.Waypoint,
    )


class RouteMatrixElement(proto.Message):
    r"""Contains route information computed for an origin/destination
    pair in the ComputeRouteMatrix API. This proto can be streamed
    to the client.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        origin_index (int):
            Zero-based index of the origin in the
            request.

            This field is a member of `oneof`_ ``_origin_index``.
        destination_index (int):
            Zero-based index of the destination in the
            request.

            This field is a member of `oneof`_ ``_destination_index``.
        status (google.rpc.status_pb2.Status):
            Error status code for this element.
        condition (google.maps.routing_v2.types.RouteMatrixElementCondition):
            Indicates whether the route was found or not.
            Independent of status.
        distance_meters (int):
            The travel distance of the route, in meters.
        duration (google.protobuf.duration_pb2.Duration):
            The length of time needed to navigate the route. If you set
            the
            [routing_preference][google.maps.routing.v2.ComputeRouteMatrixRequest.routing_preference]
            to ``TRAFFIC_UNAWARE``, then this value is the same as
            ``static_duration``. If you set the ``routing_preference``
            to either ``TRAFFIC_AWARE`` or ``TRAFFIC_AWARE_OPTIMAL``,
            then this value is calculated taking traffic conditions into
            account.
        static_duration (google.protobuf.duration_pb2.Duration):
            The duration of traveling through the route
            without taking traffic conditions into
            consideration.
        travel_advisory (google.maps.routing_v2.types.RouteTravelAdvisory):
            Additional information about the route. For
            example: restriction information and toll
            information
        fallback_info (google.maps.routing_v2.types.FallbackInfo):
            In some cases when the server is not able to
            compute the route with the given preferences for
            this particular origin/destination pair, it may
            fall back to using a different mode of
            computation. When fallback mode is used, this
            field contains detailed information about the
            fallback response. Otherwise this field is
            unset.
        localized_values (google.maps.routing_v2.types.RouteMatrixElement.LocalizedValues):
            Text representations of properties of the
            ``RouteMatrixElement``.
    """

    class LocalizedValues(proto.Message):
        r"""Text representations of certain properties.

        Attributes:
            distance (google.type.localized_text_pb2.LocalizedText):
                Travel distance represented in text form.
            duration (google.type.localized_text_pb2.LocalizedText):
                Duration represented in text form taking traffic conditions
                into consideration. Note: If traffic information was not
                requested, this value is the same value as static_duration.
            static_duration (google.type.localized_text_pb2.LocalizedText):
                Duration represented in text form without
                taking traffic conditions into consideration.
            transit_fare (google.type.localized_text_pb2.LocalizedText):
                Transit fare represented in text form.
        """

        distance: localized_text_pb2.LocalizedText = proto.Field(
            proto.MESSAGE,
            number=1,
            message=localized_text_pb2.LocalizedText,
        )
        duration: localized_text_pb2.LocalizedText = proto.Field(
            proto.MESSAGE,
            number=2,
            message=localized_text_pb2.LocalizedText,
        )
        static_duration: localized_text_pb2.LocalizedText = proto.Field(
            proto.MESSAGE,
            number=3,
            message=localized_text_pb2.LocalizedText,
        )
        transit_fare: localized_text_pb2.LocalizedText = proto.Field(
            proto.MESSAGE,
            number=4,
            message=localized_text_pb2.LocalizedText,
        )

    origin_index: int = proto.Field(
        proto.INT32,
        number=1,
        optional=True,
    )
    destination_index: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )
    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    condition: "RouteMatrixElementCondition" = proto.Field(
        proto.ENUM,
        number=9,
        enum="RouteMatrixElementCondition",
    )
    distance_meters: int = proto.Field(
        proto.INT32,
        number=4,
    )
    duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=5,
        message=duration_pb2.Duration,
    )
    static_duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=6,
        message=duration_pb2.Duration,
    )
    travel_advisory: route.RouteTravelAdvisory = proto.Field(
        proto.MESSAGE,
        number=7,
        message=route.RouteTravelAdvisory,
    )
    fallback_info: gmr_fallback_info.FallbackInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        message=gmr_fallback_info.FallbackInfo,
    )
    localized_values: LocalizedValues = proto.Field(
        proto.MESSAGE,
        number=10,
        message=LocalizedValues,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.type import localized_text_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "LocalizedTime",
    },
)


class LocalizedTime(proto.Message):
    r"""Localized description of time.

    Attributes:
        time (google.type.localized_text_pb2.LocalizedText):
            The time specified as a string in a given
            time zone.
        time_zone (str):
            Contains the time zone. The value is the name of the time
            zone as defined in the `IANA Time Zone
            Database <http://www.iana.org/time-zones>`__, e.g.
            "America/New_York".
    """

    time: localized_text_pb2.LocalizedText = proto.Field(
        proto.MESSAGE,
        number=1,
        message=localized_text_pb2.LocalizedText,
    )
    time_zone: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "Maneuver",
    },
)


class Maneuver(proto.Enum):
    r"""A set of values that specify the navigation action to take
    for the current step (for example, turn left, merge, or
    straight).

    Values:
        MANEUVER_UNSPECIFIED (0):
            Not used.
        TURN_SLIGHT_LEFT (1):
            Turn slightly to the left.
        TURN_SHARP_LEFT (2):
            Turn sharply to the left.
        UTURN_LEFT (3):
            Make a left u-turn.
        TURN_LEFT (4):
            Turn left.
        TURN_SLIGHT_RIGHT (5):
            Turn slightly to the right.
        TURN_SHARP_RIGHT (6):
            Turn sharply to the right.
        UTURN_RIGHT (7):
            Make a right u-turn.
        TURN_RIGHT (8):
            Turn right.
        STRAIGHT (9):
            Go straight.
        RAMP_LEFT (10):
            Take the left ramp.
        RAMP_RIGHT (11):
            Take the right ramp.
        MERGE (12):
            Merge into traffic.
        FORK_LEFT (13):
            Take the left fork.
        FORK_RIGHT (14):
            Take the right fork.
        FERRY (15):
            Take the ferry.
        FERRY_TRAIN (16):
            Take the train leading onto the ferry.
        ROUNDABOUT_LEFT (17):
            Turn left at the roundabout.
        ROUNDABOUT_RIGHT (18):
            Turn right at the roundabout.
        DEPART (19):
            Initial maneuver.
        NAME_CHANGE (20):
            Used to indicate a street name change.
    """
    MANEUVER_UNSPECIFIED = 0
    TURN_SLIGHT_LEFT = 1
    TURN_SHARP_LEFT = 2
    UTURN_LEFT = 3
    TURN_LEFT = 4
    TURN_SLIGHT_RIGHT = 5
    TURN_SHARP_RIGHT = 6
    UTURN_RIGHT = 7
    TURN_RIGHT = 8
    STRAIGHT = 9
    RAMP_LEFT = 10
    RAMP_RIGHT = 11
    MERGE = 12
    FORK_LEFT = 13
    FORK_RIGHT = 14
    FERRY = 15
    FERRY_TRAIN = 16
    ROUNDABOUT_LEFT = 17
    ROUNDABOUT_RIGHT = 18
    DEPART = 19
    NAME_CHANGE = 20


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import (
    AsyncIterable,
    Awaitable,
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.maps.routing_v2 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.protobuf import duration_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore

from google.maps.routing_v2.types import (
    fallback_info,
    geocoding_results,
    route,
    routes_service,
)

from .client import RoutesClient
from .transports.base import DEFAULT_CLIENT_INFO, RoutesTransport
from .transports.grpc_asyncio import RoutesGrpcAsyncIOTransport


class RoutesAsyncClient:
    """The Routes API."""

    _client: RoutesClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = RoutesClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = RoutesClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = RoutesClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = RoutesClient._DEFAULT_UNIVERSE

    common_billing_account_path = staticmethod(RoutesClient.common_billing_account_path)
    parse_common_billing_account_path = staticmethod(
        RoutesClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(RoutesClient.common_folder_path)
    parse_common_folder_path = staticmethod(RoutesClient.parse_common_folder_path)
    common_organization_path = staticmethod(RoutesClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        RoutesClient.parse_common_organization_path
    )
    common_project_path = staticmethod(RoutesClient.common_project_path)
    parse_common_project_path = staticmethod(RoutesClient.parse_common_project_path)
    common_location_path = staticmethod(RoutesClient.common_location_path)
    parse_common_location_path = staticmethod(RoutesClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            RoutesAsyncClient: The constructed client.
        """
        return RoutesClient.from_service_account_info.__func__(RoutesAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            RoutesAsyncClient: The constructed client.
        """
        return RoutesClient.from_service_account_file.__func__(RoutesAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return RoutesClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> RoutesTransport:
        """Returns the transport used by the client instance.

        Returns:
            RoutesTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = functools.partial(
        type(RoutesClient).get_transport_class, type(RoutesClient)
    )

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, RoutesTransport, Callable[..., RoutesTransport]]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the routes async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,RoutesTransport,Callable[..., RoutesTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the RoutesTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = RoutesClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def compute_routes(
        self,
        request: Optional[Union[routes_service.ComputeRoutesRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> routes_service.ComputeRoutesResponse:
        r"""Returns the primary route along with optional alternate routes,
        given a set of terminal and intermediate waypoints.

        **NOTE:** This method requires that you specify a response field
        mask in the input. You can provide the response field mask by
        using URL parameter ``$fields`` or ``fields``, or by using an
        HTTP/gRPC header ``X-Goog-FieldMask`` (see the `available URL
        parameters and
        headers <https://cloud.google.com/apis/docs/system-parameters>`__).
        The value is a comma separated list of field paths. See detailed
        documentation about `how to construct the field
        paths <https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/field_mask.proto>`__.

        For example, in this method:

        -  Field mask of all available fields (for manual inspection):
           ``X-Goog-FieldMask: *``
        -  Field mask of Route-level duration, distance, and polyline
           (an example production setup):
           ``X-Goog-FieldMask: routes.duration,routes.distanceMeters,routes.polyline.encodedPolyline``

        Google discourage the use of the wildcard (``*``) response field
        mask, or specifying the field mask at the top level
        (``routes``), because:

        -  Selecting only the fields that you need helps our server save
           computation cycles, allowing us to return the result to you
           with a lower latency.
        -  Selecting only the fields that you need in your production
           job ensures stable latency performance. We might add more
           response fields in the future, and those new fields might
           require extra computation time. If you select all fields, or
           if you select all fields at the top level, then you might
           experience performance degradation because any new field we
           add will be automatically included in the response.
        -  Selecting only the fields that you need results in a smaller
           response size, and thus higher network throughput.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import routing_v2

            async def sample_compute_routes():
                # Create a client
                client = routing_v2.RoutesAsyncClient()

                # Initialize request argument(s)
                request = routing_v2.ComputeRoutesRequest(
                )

                # Make the request
                response = await client.compute_routes(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.maps.routing_v2.types.ComputeRoutesRequest, dict]]):
                The request object. ComputeRoutes request message.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.maps.routing_v2.types.ComputeRoutesResponse:
                ComputeRoutes the response message.
        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, routes_service.ComputeRoutesRequest):
            request = routes_service.ComputeRoutesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.compute_routes
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def compute_route_matrix(
        self,
        request: Optional[Union[routes_service.ComputeRouteMatrixRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> Awaitable[AsyncIterable[routes_service.RouteMatrixElement]]:
        r"""Takes in a list of origins and destinations and returns a stream
        containing route information for each combination of origin and
        destination.

        **NOTE:** This method requires that you specify a response field
        mask in the input. You can provide the response field mask by
        using the URL parameter ``$fields`` or ``fields``, or by using
        the HTTP/gRPC header ``X-Goog-FieldMask`` (see the `available
        URL parameters and
        headers <https://cloud.google.com/apis/docs/system-parameters>`__).
        The value is a comma separated list of field paths. See this
        detailed documentation about `how to construct the field
        paths <https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/field_mask.proto>`__.

        For example, in this method:

        -  Field mask of all available fields (for manual inspection):
           ``X-Goog-FieldMask: *``
        -  Field mask of route durations, distances, element status,
           condition, and element indices (an example production setup):
           ``X-Goog-FieldMask: originIndex,destinationIndex,status,condition,distanceMeters,duration``

        It is critical that you include ``status`` in your field mask as
        otherwise all messages will appear to be OK. Google discourages
        the use of the wildcard (``*``) response field mask, because:

        -  Selecting only the fields that you need helps our server save
           computation cycles, allowing us to return the result to you
           with a lower latency.
        -  Selecting only the fields that you need in your production
           job ensures stable latency performance. We might add more
           response fields in the future, and those new fields might
           require extra computation time. If you select all fields, or
           if you select all fields at the top level, then you might
           experience performance degradation because any new field we
           add will be automatically included in the response.
        -  Selecting only the fields that you need results in a smaller
           response size, and thus higher network throughput.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import routing_v2

            async def sample_compute_route_matrix():
                # Create a client
                client = routing_v2.RoutesAsyncClient()

                # Initialize request argument(s)
                request = routing_v2.ComputeRouteMatrixRequest(
                )

                # Make the request
                stream = await client.compute_route_matrix(request=request)

                # Handle the response
                async for response in stream:
                    print(response)

        Args:
            request (Optional[Union[google.maps.routing_v2.types.ComputeRouteMatrixRequest, dict]]):
                The request object. ComputeRouteMatrix request message
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            AsyncIterable[google.maps.routing_v2.types.RouteMatrixElement]:
                Contains route information computed
                for an origin/destination pair in the
                ComputeRouteMatrix API. This proto can
                be streamed to the client.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, routes_service.ComputeRouteMatrixRequest):
            request = routes_service.ComputeRouteMatrixRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.compute_route_matrix
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "RoutesAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("RoutesAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import os
import re
from typing import (
    Callable,
    Dict,
    Iterable,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    cast,
)
import warnings

from google.api_core import client_options as client_options_lib
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.maps.routing_v2 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

from google.protobuf import duration_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore

from google.maps.routing_v2.types import (
    fallback_info,
    geocoding_results,
    route,
    routes_service,
)

from .transports.base import DEFAULT_CLIENT_INFO, RoutesTransport
from .transports.grpc import RoutesGrpcTransport
from .transports.grpc_asyncio import RoutesGrpcAsyncIOTransport
from .transports.rest import RoutesRestTransport


class RoutesClientMeta(type):
    """Metaclass for the Routes client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = OrderedDict()  # type: Dict[str, Type[RoutesTransport]]
    _transport_registry["grpc"] = RoutesGrpcTransport
    _transport_registry["grpc_asyncio"] = RoutesGrpcAsyncIOTransport
    _transport_registry["rest"] = RoutesRestTransport

    def get_transport_class(
        cls,
        label: Optional[str] = None,
    ) -> Type[RoutesTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class RoutesClient(metaclass=RoutesClientMeta):
    """The Routes API."""

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = "routes.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    _DEFAULT_ENDPOINT_TEMPLATE = "routes.{UNIVERSE_DOMAIN}"
    _DEFAULT_UNIVERSE = "googleapis.com"

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            RoutesClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(info)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            RoutesClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> RoutesTransport:
        """Returns the transport used by the client instance.

        Returns:
            RoutesTransport: The transport used by the client
                instance.
        """
        return self._transport

    @staticmethod
    def common_billing_account_path(
        billing_account: str,
    ) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(
        folder: str,
    ) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(
            folder=folder,
        )

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(
        organization: str,
    ) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(
            organization=organization,
        )

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(
        project: str,
    ) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(
            project=project,
        )

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(
        project: str,
        location: str,
    ) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project,
            location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
        return m.groupdict() if m else {}

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[client_options_lib.ClientOptions] = None
    ):
        """Deprecated. Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """

        warnings.warn(
            "get_mtls_endpoint_and_cert_source is deprecated. Use the api_endpoint property instead.",
            DeprecationWarning,
        )
        if client_options is None:
            client_options = client_options_lib.ClientOptions()
        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )

        # Figure out the client cert source to use.
        client_cert_source = None
        if use_client_cert == "true":
            if client_options.client_cert_source:
                client_cert_source = client_options.client_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = cls.DEFAULT_ENDPOINT

        return api_endpoint, client_cert_source

    @staticmethod
    def _read_environment_variables():
        """Returns the environment variables used by the client.

        Returns:
            Tuple[bool, str, str]: returns the GOOGLE_API_USE_CLIENT_CERTIFICATE,
            GOOGLE_API_USE_MTLS_ENDPOINT, and GOOGLE_CLOUD_UNIVERSE_DOMAIN environment variables.

        Raises:
            ValueError: If GOOGLE_API_USE_CLIENT_CERTIFICATE is not
                any of ["true", "false"].
            google.auth.exceptions.MutualTLSChannelError: If GOOGLE_API_USE_MTLS_ENDPOINT
                is not any of ["auto", "never", "always"].
        """
        use_client_cert = os.getenv(
            "GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"
        ).lower()
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto").lower()
        universe_domain_env = os.getenv("GOOGLE_CLOUD_UNIVERSE_DOMAIN")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )
        return use_client_cert == "true", use_mtls_endpoint, universe_domain_env

    @staticmethod
    def _get_client_cert_source(provided_cert_source, use_cert_flag):
        """Return the client cert source to be used by the client.

        Args:
            provided_cert_source (bytes): The client certificate source provided.
            use_cert_flag (bool): A flag indicating whether to use the client certificate.

        Returns:
            bytes or None: The client cert source to be used by the client.
        """
        client_cert_source = None
        if use_cert_flag:
            if provided_cert_source:
                client_cert_source = provided_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()
        return client_cert_source

    @staticmethod
    def _get_api_endpoint(
        api_override, client_cert_source, universe_domain, use_mtls_endpoint
    ):
        """Return the API endpoint used by the client.

        Args:
            api_override (str): The API endpoint override. If specified, this is always
                the return value of this function and the other arguments are not used.
            client_cert_source (bytes): The client certificate source used by the client.
            universe_domain (str): The universe domain used by the client.
            use_mtls_endpoint (str): How to use the mTLS endpoint, which depends also on the other parameters.
                Possible values are "always", "auto", or "never".

        Returns:
            str: The API endpoint to be used by the client.
        """
        if api_override is not None:
            api_endpoint = api_override
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            _default_universe = RoutesClient._DEFAULT_UNIVERSE
            if universe_domain != _default_universe:
                raise MutualTLSChannelError(
                    f"mTLS is not supported in any universe other than {_default_universe}."
                )
            api_endpoint = RoutesClient.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = RoutesClient._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=universe_domain
            )
        return api_endpoint

    @staticmethod
    def _get_universe_domain(
        client_universe_domain: Optional[str], universe_domain_env: Optional[str]
    ) -> str:
        """Return the universe domain used by the client.

        Args:
            client_universe_domain (Optional[str]): The universe domain configured via the client options.
            universe_domain_env (Optional[str]): The universe domain configured via the "GOOGLE_CLOUD_UNIVERSE_DOMAIN" environment variable.

        Returns:
            str: The universe domain to be used by the client.

        Raises:
            ValueError: If the universe domain is an empty string.
        """
        universe_domain = RoutesClient._DEFAULT_UNIVERSE
        if client_universe_domain is not None:
            universe_domain = client_universe_domain
        elif universe_domain_env is not None:
            universe_domain = universe_domain_env
        if len(universe_domain.strip()) == 0:
            raise ValueError("Universe Domain cannot be an empty string.")
        return universe_domain

    @staticmethod
    def _compare_universes(
        client_universe: str, credentials: ga_credentials.Credentials
    ) -> bool:
        """Returns True iff the universe domains used by the client and credentials match.

        Args:
            client_universe (str): The universe domain configured via the client options.
            credentials (ga_credentials.Credentials): The credentials being used in the client.

        Returns:
            bool: True iff client_universe matches the universe in credentials.

        Raises:
            ValueError: when client_universe does not match the universe in credentials.
        """

        default_universe = RoutesClient._DEFAULT_UNIVERSE
        credentials_universe = getattr(credentials, "universe_domain", default_universe)

        if client_universe != credentials_universe:
            raise ValueError(
                "The configured universe domain "
                f"({client_universe}) does not match the universe domain "
                f"found in the credentials ({credentials_universe}). "
                "If you haven't configured the universe domain explicitly, "
                f"`{default_universe}` is the default."
            )
        return True

    def _validate_universe_domain(self):
        """Validates client's and credentials' universe domains are consistent.

        Returns:
            bool: True iff the configured universe domain is valid.

        Raises:
            ValueError: If the configured universe domain is not valid.
        """
        self._is_universe_domain_valid = (
            self._is_universe_domain_valid
            or RoutesClient._compare_universes(
                self.universe_domain, self.transport._credentials
            )
        )
        return self._is_universe_domain_valid

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used by the client instance.
        """
        return self._universe_domain

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, RoutesTransport, Callable[..., RoutesTransport]]
        ] = None,
        client_options: Optional[Union[client_options_lib.ClientOptions, dict]] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the routes client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,RoutesTransport,Callable[..., RoutesTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the RoutesTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that the ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client_options = client_options
        if isinstance(self._client_options, dict):
            self._client_options = client_options_lib.from_dict(self._client_options)
        if self._client_options is None:
            self._client_options = client_options_lib.ClientOptions()
        self._client_options = cast(
            client_options_lib.ClientOptions, self._client_options
        )

        universe_domain_opt = getattr(self._client_options, "universe_domain", None)

        (
            self._use_client_cert,
            self._use_mtls_endpoint,
            self._universe_domain_env,
        ) = RoutesClient._read_environment_variables()
        self._client_cert_source = RoutesClient._get_client_cert_source(
            self._client_options.client_cert_source, self._use_client_cert
        )
        self._universe_domain = RoutesClient._get_universe_domain(
            universe_domain_opt, self._universe_domain_env
        )
        self._api_endpoint = None  # updated below, depending on `transport`

        # Initialize the universe domain validation.
        self._is_universe_domain_valid = False

        api_key_value = getattr(self._client_options, "api_key", None)
        if api_key_value and credentials:
            raise ValueError(
                "client_options.api_key and credentials are mutually exclusive"
            )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        transport_provided = isinstance(transport, RoutesTransport)
        if transport_provided:
            # transport is a RoutesTransport instance.
            if credentials or self._client_options.credentials_file or api_key_value:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if self._client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = cast(RoutesTransport, transport)
            self._api_endpoint = self._transport.host

        self._api_endpoint = self._api_endpoint or RoutesClient._get_api_endpoint(
            self._client_options.api_endpoint,
            self._client_cert_source,
            self._universe_domain,
            self._use_mtls_endpoint,
        )

        if not transport_provided:
            import google.auth._default  # type: ignore

            if api_key_value and hasattr(
                google.auth._default, "get_api_key_credentials"
            ):
                credentials = google.auth._default.get_api_key_credentials(
                    api_key_value
                )

            transport_init: Union[
                Type[RoutesTransport], Callable[..., RoutesTransport]
            ] = (
                type(self).get_transport_class(transport)
                if isinstance(transport, str) or transport is None
                else cast(Callable[..., RoutesTransport], transport)
            )
            # initialize with the provided callable or the passed in class
            self._transport = transport_init(
                credentials=credentials,
                credentials_file=self._client_options.credentials_file,
                host=self._api_endpoint,
                scopes=self._client_options.scopes,
                client_cert_source_for_mtls=self._client_cert_source,
                quota_project_id=self._client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
                api_audience=self._client_options.api_audience,
            )

    def compute_routes(
        self,
        request: Optional[Union[routes_service.ComputeRoutesRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> routes_service.ComputeRoutesResponse:
        r"""Returns the primary route along with optional alternate routes,
        given a set of terminal and intermediate waypoints.

        **NOTE:** This method requires that you specify a response field
        mask in the input. You can provide the response field mask by
        using URL parameter ``$fields`` or ``fields``, or by using an
        HTTP/gRPC header ``X-Goog-FieldMask`` (see the `available URL
        parameters and
        headers <https://cloud.google.com/apis/docs/system-parameters>`__).
        The value is a comma separated list of field paths. See detailed
        documentation about `how to construct the field
        paths <https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/field_mask.proto>`__.

        For example, in this method:

        -  Field mask of all available fields (for manual inspection):
           ``X-Goog-FieldMask: *``
        -  Field mask of Route-level duration, distance, and polyline
           (an example production setup):
           ``X-Goog-FieldMask: routes.duration,routes.distanceMeters,routes.polyline.encodedPolyline``

        Google discourage the use of the wildcard (``*``) response field
        mask, or specifying the field mask at the top level
        (``routes``), because:

        -  Selecting only the fields that you need helps our server save
           computation cycles, allowing us to return the result to you
           with a lower latency.
        -  Selecting only the fields that you need in your production
           job ensures stable latency performance. We might add more
           response fields in the future, and those new fields might
           require extra computation time. If you select all fields, or
           if you select all fields at the top level, then you might
           experience performance degradation because any new field we
           add will be automatically included in the response.
        -  Selecting only the fields that you need results in a smaller
           response size, and thus higher network throughput.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import routing_v2

            def sample_compute_routes():
                # Create a client
                client = routing_v2.RoutesClient()

                # Initialize request argument(s)
                request = routing_v2.ComputeRoutesRequest(
                )

                # Make the request
                response = client.compute_routes(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.maps.routing_v2.types.ComputeRoutesRequest, dict]):
                The request object. ComputeRoutes request message.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.maps.routing_v2.types.ComputeRoutesResponse:
                ComputeRoutes the response message.
        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, routes_service.ComputeRoutesRequest):
            request = routes_service.ComputeRoutesRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.compute_routes]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def compute_route_matrix(
        self,
        request: Optional[Union[routes_service.ComputeRouteMatrixRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> Iterable[routes_service.RouteMatrixElement]:
        r"""Takes in a list of origins and destinations and returns a stream
        containing route information for each combination of origin and
        destination.

        **NOTE:** This method requires that you specify a response field
        mask in the input. You can provide the response field mask by
        using the URL parameter ``$fields`` or ``fields``, or by using
        the HTTP/gRPC header ``X-Goog-FieldMask`` (see the `available
        URL parameters and
        headers <https://cloud.google.com/apis/docs/system-parameters>`__).
        The value is a comma separated list of field paths. See this
        detailed documentation about `how to construct the field
        paths <https://github.com/protocolbuffers/protobuf/blob/master/src/google/protobuf/field_mask.proto>`__.

        For example, in this method:

        -  Field mask of all available fields (for manual inspection):
           ``X-Goog-FieldMask: *``
        -  Field mask of route durations, distances, element status,
           condition, and element indices (an example production setup):
           ``X-Goog-FieldMask: originIndex,destinationIndex,status,condition,distanceMeters,duration``

        It is critical that you include ``status`` in your field mask as
        otherwise all messages will appear to be OK. Google discourages
        the use of the wildcard (``*``) response field mask, because:

        -  Selecting only the fields that you need helps our server save
           computation cycles, allowing us to return the result to you
           with a lower latency.
        -  Selecting only the fields that you need in your production
           job ensures stable latency performance. We might add more
           response fields in the future, and those new fields might
           require extra computation time. If you select all fields, or
           if you select all fields at the top level, then you might
           experience performance degradation because any new field we
           add will be automatically included in the response.
        -  Selecting only the fields that you need results in a smaller
           response size, and thus higher network throughput.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import routing_v2

            def sample_compute_route_matrix():
                # Create a client
                client = routing_v2.RoutesClient()

                # Initialize request argument(s)
                request = routing_v2.ComputeRouteMatrixRequest(
                )

                # Make the request
                stream = client.compute_route_matrix(request=request)

                # Handle the response
                for response in stream:
                    print(response)

        Args:
            request (Union[google.maps.routing_v2.types.ComputeRouteMatrixRequest, dict]):
                The request object. ComputeRouteMatrix request message
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            Iterable[google.maps.routing_v2.types.RouteMatrixElement]:
                Contains route information computed
                for an origin/destination pair in the
                ComputeRouteMatrix API. This proto can
                be streamed to the client.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, routes_service.ComputeRouteMatrixRequest):
            request = routes_service.ComputeRouteMatrixRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.compute_route_matrix]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def __enter__(self) -> "RoutesClient":
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("RoutesClient",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.maps.routing_v2.types import location as gmr_location

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "Waypoint",
    },
)


class Waypoint(proto.Message):
    r"""Encapsulates a waypoint. Waypoints mark both the beginning
    and end of a route, and include intermediate stops along the
    route.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        location (google.maps.routing_v2.types.Location):
            A point specified using geographic
            coordinates, including an optional heading.

            This field is a member of `oneof`_ ``location_type``.
        place_id (str):
            The POI Place ID associated with the
            waypoint.

            This field is a member of `oneof`_ ``location_type``.
        address (str):
            Human readable address or a plus code.
            See https://plus.codes for details.

            This field is a member of `oneof`_ ``location_type``.
        via (bool):
            Marks this waypoint as a milestone rather a stopping point.
            For each non-via waypoint in the request, the response
            appends an entry to the
            [``legs``][google.maps.routing.v2.Route.legs] array to
            provide the details for stopovers on that leg of the trip.
            Set this value to true when you want the route to pass
            through this waypoint without stopping over. Via waypoints
            don't cause an entry to be added to the ``legs`` array, but
            they do route the journey through the waypoint. You can only
            set this value on waypoints that are intermediates. The
            request fails if you set this field on terminal waypoints.
            If ``ComputeRoutesRequest.optimize_waypoint_order`` is set
            to true then this field cannot be set to true; otherwise,
            the request fails.
        vehicle_stopover (bool):
            Indicates that the waypoint is meant for vehicles to stop
            at, where the intention is to either pickup or drop-off.
            When you set this value, the calculated route won't include
            non-\ ``via`` waypoints on roads that are unsuitable for
            pickup and drop-off. This option works only for ``DRIVE``
            and ``TWO_WHEELER`` travel modes, and when the
            ``location_type`` is
            [``Location``][google.maps.routing.v2.Location].
        side_of_road (bool):
            Indicates that the location of this waypoint is meant to
            have a preference for the vehicle to stop at a particular
            side of road. When you set this value, the route will pass
            through the location so that the vehicle can stop at the
            side of road that the location is biased towards from the
            center of the road. This option works only for ``DRIVE`` and
            ``TWO_WHEELER``
            [``RouteTravelMode``][google.maps.routing.v2.RouteTravelMode].
    """

    location: gmr_location.Location = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="location_type",
        message=gmr_location.Location,
    )
    place_id: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="location_type",
    )
    address: str = proto.Field(
        proto.STRING,
        number=7,
        oneof="location_type",
    )
    via: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    vehicle_stopover: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    side_of_road: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "RouteTravelMode",
    },
)


class RouteTravelMode(proto.Enum):
    r"""A set of values used to specify the mode of travel. NOTE: ``WALK``,
    ``BICYCLE``, and ``TWO_WHEELER`` routes are in beta and might
    sometimes be missing clear sidewalks, pedestrian paths, or bicycling
    paths. You must display this warning to the user for all walking,
    bicycling, and two-wheel routes that you display in your app.

    Values:
        TRAVEL_MODE_UNSPECIFIED (0):
            No travel mode specified. Defaults to ``DRIVE``.
        DRIVE (1):
            Travel by passenger car.
        BICYCLE (2):
            Travel by bicycle.
        WALK (3):
            Travel by walking.
        TWO_WHEELER (4):
            Two-wheeled, motorized vehicle. For example, motorcycle.
            Note that this differs from the ``BICYCLE`` travel mode
            which covers human-powered mode.
        TRANSIT (7):
            Travel by public transit routes, where
            available.
    """
    TRAVEL_MODE_UNSPECIFIED = 0
    DRIVE = 1
    BICYCLE = 2
    WALK = 3
    TWO_WHEELER = 4
    TRANSIT = 7


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .fallback_info import FallbackInfo, FallbackReason, FallbackRoutingMode
from .geocoding_results import GeocodedWaypoint, GeocodingResults
from .localized_time import LocalizedTime
from .location import Location
from .maneuver import Maneuver
from .navigation_instruction import NavigationInstruction
from .polyline import Polyline, PolylineEncoding, PolylineQuality
from .route import (
    Route,
    RouteLeg,
    RouteLegStep,
    RouteLegStepTransitDetails,
    RouteLegStepTravelAdvisory,
    RouteLegTravelAdvisory,
    RouteTravelAdvisory,
)
from .route_label import RouteLabel
from .route_modifiers import RouteModifiers
from .route_travel_mode import RouteTravelMode
from .routes_service import (
    ComputeRouteMatrixRequest,
    ComputeRoutesRequest,
    ComputeRoutesResponse,
    RouteMatrixDestination,
    RouteMatrixElement,
    RouteMatrixElementCondition,
    RouteMatrixOrigin,
)
from .routing_preference import RoutingPreference
from .speed_reading_interval import SpeedReadingInterval
from .toll_info import TollInfo
from .toll_passes import TollPass
from .traffic_model import TrafficModel
from .transit import TransitAgency, TransitLine, TransitStop, TransitVehicle
from .transit_preferences import TransitPreferences
from .units import Units
from .vehicle_emission_type import VehicleEmissionType
from .vehicle_info import VehicleInfo
from .waypoint import Waypoint

__all__ = (
    "FallbackInfo",
    "FallbackReason",
    "FallbackRoutingMode",
    "GeocodedWaypoint",
    "GeocodingResults",
    "LocalizedTime",
    "Location",
    "Maneuver",
    "NavigationInstruction",
    "Polyline",
    "PolylineEncoding",
    "PolylineQuality",
    "Route",
    "RouteLeg",
    "RouteLegStep",
    "RouteLegStepTransitDetails",
    "RouteLegStepTravelAdvisory",
    "RouteLegTravelAdvisory",
    "RouteTravelAdvisory",
    "RouteLabel",
    "RouteModifiers",
    "RouteTravelMode",
    "ComputeRouteMatrixRequest",
    "ComputeRoutesRequest",
    "ComputeRoutesResponse",
    "RouteMatrixDestination",
    "RouteMatrixElement",
    "RouteMatrixOrigin",
    "RouteMatrixElementCondition",
    "RoutingPreference",
    "SpeedReadingInterval",
    "TollInfo",
    "TollPass",
    "TrafficModel",
    "TransitAgency",
    "TransitLine",
    "TransitStop",
    "TransitVehicle",
    "TransitPreferences",
    "Units",
    "VehicleEmissionType",
    "VehicleInfo",
    "Waypoint",
)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import struct_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "PolylineQuality",
        "PolylineEncoding",
        "Polyline",
    },
)


class PolylineQuality(proto.Enum):
    r"""A set of values that specify the quality of the polyline.

    Values:
        POLYLINE_QUALITY_UNSPECIFIED (0):
            No polyline quality preference specified. Defaults to
            ``OVERVIEW``.
        HIGH_QUALITY (1):
            Specifies a high-quality polyline - which is composed using
            more points than ``OVERVIEW``, at the cost of increased
            response size. Use this value when you need more precision.
        OVERVIEW (2):
            Specifies an overview polyline - which is composed using a
            small number of points. Use this value when displaying an
            overview of the route. Using this option has a lower request
            latency compared to using the ``HIGH_QUALITY`` option.
    """
    POLYLINE_QUALITY_UNSPECIFIED = 0
    HIGH_QUALITY = 1
    OVERVIEW = 2


class PolylineEncoding(proto.Enum):
    r"""Specifies the preferred type of polyline to be returned.

    Values:
        POLYLINE_ENCODING_UNSPECIFIED (0):
            No polyline type preference specified. Defaults to
            ``ENCODED_POLYLINE``.
        ENCODED_POLYLINE (1):
            Specifies a polyline encoded using the `polyline encoding
            algorithm </maps/documentation/utilities/polylinealgorithm>`__.
        GEO_JSON_LINESTRING (2):
            Specifies a polyline using the `GeoJSON LineString
            format <https://tools.ietf.org/html/rfc7946#section-3.1.4>`__
    """
    POLYLINE_ENCODING_UNSPECIFIED = 0
    ENCODED_POLYLINE = 1
    GEO_JSON_LINESTRING = 2


class Polyline(proto.Message):
    r"""Encapsulates an encoded polyline.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        encoded_polyline (str):
            The string encoding of the polyline using the `polyline
            encoding
            algorithm <https://developers.google.com/maps/documentation/utilities/polylinealgorithm>`__

            This field is a member of `oneof`_ ``polyline_type``.
        geo_json_linestring (google.protobuf.struct_pb2.Struct):
            Specifies a polyline using the `GeoJSON LineString
            format <https://tools.ietf.org/html/rfc7946#section-3.1.4>`__.

            This field is a member of `oneof`_ ``polyline_type``.
    """

    encoded_polyline: str = proto.Field(
        proto.STRING,
        number=1,
        oneof="polyline_type",
    )
    geo_json_linestring: struct_pb2.Struct = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="polyline_type",
        message=struct_pb2.Struct,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.rpc import status_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "GeocodingResults",
        "GeocodedWaypoint",
    },
)


class GeocodingResults(proto.Message):
    r"""Contains
    [``GeocodedWaypoints``][google.maps.routing.v2.GeocodedWaypoint] for
    origin, destination and intermediate waypoints. Only populated for
    address waypoints.

    Attributes:
        origin (google.maps.routing_v2.types.GeocodedWaypoint):
            Origin geocoded waypoint.
        destination (google.maps.routing_v2.types.GeocodedWaypoint):
            Destination geocoded waypoint.
        intermediates (MutableSequence[google.maps.routing_v2.types.GeocodedWaypoint]):
            A list of intermediate geocoded waypoints
            each containing an index field that corresponds
            to the zero-based position of the waypoint in
            the order they were specified in the request.
    """

    origin: "GeocodedWaypoint" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="GeocodedWaypoint",
    )
    destination: "GeocodedWaypoint" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="GeocodedWaypoint",
    )
    intermediates: MutableSequence["GeocodedWaypoint"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="GeocodedWaypoint",
    )


class GeocodedWaypoint(proto.Message):
    r"""Details about the locations used as waypoints. Only populated
    for address waypoints. Includes details about the geocoding
    results for the purposes of determining what the address was
    geocoded to.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        geocoder_status (google.rpc.status_pb2.Status):
            Indicates the status code resulting from the
            geocoding operation.
        intermediate_waypoint_request_index (int):
            The index of the corresponding intermediate
            waypoint in the request. Only populated if the
            corresponding waypoint is an intermediate
            waypoint.

            This field is a member of `oneof`_ ``_intermediate_waypoint_request_index``.
        type_ (MutableSequence[str]):
            The type(s) of the result, in the form of zero or more type
            tags. Supported types: `Address types and address component
            types <https://developers.google.com/maps/documentation/geocoding/requests-geocoding#Types>`__.
        partial_match (bool):
            Indicates that the geocoder did not return an
            exact match for the original request, though it
            was able to match part of the requested address.
            You may wish to examine the original request for
            misspellings and/or an incomplete address.
        place_id (str):
            The place ID for this result.
    """

    geocoder_status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    intermediate_waypoint_request_index: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )
    type_: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )
    partial_match: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    place_id: str = proto.Field(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "TransitPreferences",
    },
)


class TransitPreferences(proto.Message):
    r"""Preferences for ``TRANSIT`` based routes that influence the route
    that is returned.

    Attributes:
        allowed_travel_modes (MutableSequence[google.maps.routing_v2.types.TransitPreferences.TransitTravelMode]):
            A set of travel modes to use when getting a ``TRANSIT``
            route. Defaults to all supported modes of travel.
        routing_preference (google.maps.routing_v2.types.TransitPreferences.TransitRoutingPreference):
            A routing preference that, when specified, influences the
            ``TRANSIT`` route returned.
    """

    class TransitTravelMode(proto.Enum):
        r"""A set of values used to specify the mode of transit.

        Values:
            TRANSIT_TRAVEL_MODE_UNSPECIFIED (0):
                No transit travel mode specified.
            BUS (1):
                Travel by bus.
            SUBWAY (2):
                Travel by subway.
            TRAIN (3):
                Travel by train.
            LIGHT_RAIL (4):
                Travel by light rail or tram.
            RAIL (5):
                Travel by rail. This is equivalent to a combination of
                ``SUBWAY``, ``TRAIN``, and ``LIGHT_RAIL``.
        """
        TRANSIT_TRAVEL_MODE_UNSPECIFIED = 0
        BUS = 1
        SUBWAY = 2
        TRAIN = 3
        LIGHT_RAIL = 4
        RAIL = 5

    class TransitRoutingPreference(proto.Enum):
        r"""Specifies routing preferences for transit routes.

        Values:
            TRANSIT_ROUTING_PREFERENCE_UNSPECIFIED (0):
                No preference specified.
            LESS_WALKING (1):
                Indicates that the calculated route should
                prefer limited amounts of walking.
            FEWER_TRANSFERS (2):
                Indicates that the calculated route should
                prefer a limited number of transfers.
        """
        TRANSIT_ROUTING_PREFERENCE_UNSPECIFIED = 0
        LESS_WALKING = 1
        FEWER_TRANSFERS = 2

    allowed_travel_modes: MutableSequence[TransitTravelMode] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=TransitTravelMode,
    )
    routing_preference: TransitRoutingPreference = proto.Field(
        proto.ENUM,
        number=2,
        enum=TransitRoutingPreference,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.type import money_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.routing.v2",
    manifest={
        "TollInfo",
    },
)


class TollInfo(proto.Message):
    r"""Encapsulates toll information on a
    [``Route``][google.maps.routing.v2.Route] or on a
    [``RouteLeg``][google.maps.routing.v2.RouteLeg].

    Attributes:
        estimated_price (MutableSequence[google.type.money_pb2.Money]):
            The monetary amount of tolls for the corresponding
            [``Route``][google.maps.routing.v2.Route] or
            [``RouteLeg``][google.maps.routing.v2.RouteLeg]. This list
            contains a money amount for each currency that is expected
            to be charged by the toll stations. Typically this list will
            contain only one item for routes with tolls in one currency.
            For international trips, this list may contain multiple
            items to reflect tolls in different currencies.
    """

    estimated_price: MutableSequence[money_pb2.Money] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=money_pb2.Money,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
